/*	$Csoft	    */

/*
 * Copyright (c) 1999-2001 CubeSoft Communications, Inc.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistribution of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistribution in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. Neither the name of CubeSoft Communications, nor the names of its
 *    contributors may be used to endorse or promote products derived from
 *    this software without specific prior written permission.
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE LIABLE FOR
 * ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
 * SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
 * CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE
 * USE OF THIS SOFTWARE EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

#include <sys/types.h>
#include <sys/stat.h>
#include <unistd.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <fcntl.h>
#include <glib.h>

#include "techmlm.h"

static void	 	 techmlm_printusage(char *);
static struct techmlm	*new_techmlm(void);
static void 		 free_techmlm(struct techmlm *);
static void		 quit(struct techmlm *, char);
static void 		 techmlm_dirs(struct techmlm *);

static void 
techmlm_printusage(char *argv0)
{
	printf("Usage: %s [-rATv] [-e email] [-t dir] [-l list] "
	       "[-L log] [-a dir]\n", argv0);
}

static struct techmlm *
new_techmlm(void)
{
	struct techmlm *ti;

	ti = malloc(sizeof(struct techmlm));
	if (ti == NULL) {
		perror("malloc");
		abort();
	}
	
	ti->tf = NULL;
	ti->lf = NULL;
	ti->af = NULL;
	ti->cf = NULL;
	ti->logfile = NULL;
	ti->ardir = NULL;
	ti->ciddir = NULL;

	ti->msgid = NULL;
	ti->email = NULL;
	ti->casetype = NULL;
	ti->cid = NULL;
	ti->totlen = 0;

	ti->defer = 0;
	ti->archive = 1;
	ti->tamperwith = 1;
	ti->replyto = 0;

#ifdef USE_QMAIL
	ti->qi.sender = NULL;
	ti->qi.newsender = NULL;
	ti->qi.recipient = NULL;
	ti->qi.host = NULL;
	ti->qi.local = NULL;
#endif

	regetdate(ti);

	return (ti);
}

static void 
free_techmlm(struct techmlm *ti)
{
	if (ti->tf)
		free(ti->tf);
	if (ti->lf)
		free(ti->lf);
	if (ti->af)
		free(ti->af);
	if (ti->cf)
		free(ti->cf);
	if (ti->casetype)
		free(ti->casetype);
	if (ti->logfile)
		free(ti->logfile);
	if (ti->email)
		free(ti->email);
	if (ti->ardir)
		free(ti->ardir);
	if (ti->ciddir)
		free(ti->ciddir);
	free(ti);
}

static void 
quit(struct techmlm *ti, char status)
{
	if (!ti->defer)
		unlink(ti->tf);
	if (ti)
		free_techmlm(ti);

	switch (status) {
	case S_DEFER:
		exit(111);
	case S_FAIL:
		exit(100);
	}
	exit(0);
}

static void 
techmlm_dirs(struct techmlm *ti)
{
	struct stat sta;
	if (stat(ti->ardir, &sta) < 0) {
		mkdir(ti->ardir, 00700);
		log(ti, L_INFO, "created archive dir: %s", ti->ardir);
	}
	if (stat(ti->ciddir, &sta) < 0) {
		mkdir(ti->ciddir, 00700);
		log(ti, L_INFO, "created cases dir: %s", ti->ciddir);
	}
}

int 
main(int argc, char *argv[])
{
	char mld[BUFSIZE], truncwarn = 0;
	int tf, rb, c;
	struct techmlm *ti = new_techmlm();

	while ((c = getopt(argc, argv, "t:l:L:e:a:c:hAvTr")) != EOF) {
		switch (c) {
		case 'e':
			ti->email = strdup(optarg);
			break;
		case 't':
			ti->tf = strdup(optarg);
			break;
		case 'l':
			ti->lf = strdup(optarg);
			break;
		case 'L':
			ti->logfile = strdup(optarg);
			break;
		case 'a':
			ti->ardir = strdup(optarg);
			break;
		case 'A':
			ti->archive = 0;
			break;
		case 'T':
			ti->tamperwith = 0;
			break;
		case 'r':
			ti->replyto++;
			break;
		case 'h':
			techmlm_printusage(argv[0]);
			quit(ti, S_OK);
			break;
		case 'v':
			fprintf(stderr, "techmlm %s\n", VERSION);
			quit(ti, S_OK);
			break;
		}
	}

	if (!ti->logfile)
		ti->logfile = strdup(DEFAULT_LOGFILE);
	if (!ti->ardir)
		ti->ardir = g_strdup_printf("%s/%i-%i", DEFAULT_ARDIR,
		    ti->mon, ti->year);
	if (!ti->ciddir)
		ti->ciddir = g_strdup_printf("%s/cases", ti->ardir);

	if (!ti->tf)
		ti->tf = g_strdup_printf("%s/%i", DEFAULT_TMPDIR, getpid());
	if (!ti->lf)
		ti->lf = strdup(DEFAULT_LISTFILE);
	if (!ti->af)
		ti->af = g_strdup_printf("%s/%i.%i-%i-%i", ti->ardir,
		    ti->mday, ti->hour, ti->min, ti->sec);

	if ((tf = open(ti->tf, O_WRONLY | O_CREAT, 00600)) < 0)
		quit(ti, S_FAIL);
	ti->msgid = msgid(ti);
	ti->cf = g_strdup_printf("%s/cases/%s", ti->ardir, ti->msgid);
	techmlm_dirs(ti);

	while ((rb = read(0, mld, BUFSIZE)) > 0) {
#ifdef MAX_LENGTH
		if (ti->totlen > MAX_LENGTH) {
			if (!truncwarn) {
				log(ti, L_WARN, "truncated (size=%i, max=%i)",
				    ti->totlen, MAX_LENGTH);
				truncwarn++;
			}
			continue;
		}
#endif
		ti->totlen += write(tf, mld, rb);
	}
	close(tf);

#ifdef USE_QMAIL
	qmail_getenv(ti);
	log(ti, L_INFO, "len=%i, from %s, to %s, host=%s",
	    ti->totlen, ti->qi.newsender, ti->qi.recipient, ti->qi.host);
#endif

#ifdef TAMPERWITH
	if (ti->tamperwith)
		techmlm_tamperwith(ti);
#endif

	if (techmlm_deliver(ti) != D_FATAL) {
#ifdef ARCHIVE
		if (ti->archive)
			techmlm_archive(ti);
#endif
	} else {
		quit(ti, S_FAIL);
	}

	quit(ti, S_OK);
	return (0);
}
